## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions
# Great Statesman
# By: The Lopez

import CvUtil
import PyHelpers
import CvConfigParser

import Popup as PyPopup

from CvPythonExtensions import *

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo

DIPLOMATIC_RELATIONS_POPUP = 4203


# Change this to either enable or disable the improve diplomatics relations
# ability of the great statesmen.
# Default value is true
g_bEnableImproveDiplomaticRelations = true

# Change this to either enable or disable the start insurrection ability of the 
# great statesmen.
# Default value is true
g_bEnableGreatStatesmanStartInsurrection = true

# Change this to true if great statesmen should be allowed to start 
# insurrections in friendly cities.
# Default value is false
g_bStartInsurrectionInFriendlyCities = false

# Change this to true if great statesmen should be allowed to start 
# insurrections in player capital cities.
# Default value is false
g_bStartInsurrectionInPlayerCapitals = false

# Change this to false if insurrections should not allow cities to flip to the
# player's civilization.
# Default value is true
g_bInsurrectionCanFlipCity = true

# Increase or decrease the value to change the base amount of time the 
# insurrection will last.
# Default value is 6
g_iBaseInsurrectionTime = 6

# Change this to false if the base insurrection time should be based on the
# player's cultural influence.
# Default value is true
g_bInsurrectionTimePlayerCultureBased = true

class GreatStatesmanUtils :

	def __init__(self):
	
		global g_bEnableImproveDiplomaticRelations
		global g_bEnableGreatStatesmanStartInsurrection
		global g_bStartInsurrectionInFriendlyCities
		global g_bStartInsurrectionInPlayerCapitals
		global g_bInsurrectionCanFlipCity
		global g_iBaseInsurrectionTime
		global g_bInsurrectionTimePlayerCultureBased		
		

		config = CvConfigParser.CvConfigParser("Great Statesman Config.ini")

		# Read the configuration values for the great statesman
		if(config != None):
			g_bEnableImproveDiplomaticRelations = config.getboolean("Great Statesman", "Enable Improve Diplomatic Relations", true)

			g_bEnableGreatStatesmanStartInsurrection = config.getboolean("Great Statesman", "Enable Great Statesman Start Insurrection", true)

			g_bStartInsurrectionInFriendlyCities = config.getboolean("Great Statesman", "Start Insurrection In Friendly Cities", false)

			g_bStartInsurrectionInPlayerCapitals = config.getboolean("Great Statesman", "Start Insurrection In Player Capitals", false)
			
			g_bInsurrectionCanFlipCity = config.getboolean("Great Statesman", "Insurrection Can Flip City", true)

			g_iBaseInsurrectionTime = config.getint("Great Statesman", "Base Insurrection Time", 6)

			g_bInsurrectionTimePlayerCultureBased = config.getboolean("Great Statesman", "Insurrection Time Player Culture Based", true)


	def canImproveDiplomaticRelations(self, objUnit):

		if(not g_bEnableImproveDiplomaticRelations):
			return fales
			
		if(objUnit == None):
			return false
			
		if(objUnit.isNone()):
			return false

		if(objUnit.getUnitType() != gc.getInfoTypeForString("UNIT_STATESMAN")):
			return false
		
		objPlayerTeam = gc.getTeam(objUnit.getTeam())
		
		if(objPlayerTeam.getHasMetCivCount(True) == 0):
			return false
			
		return true
	
	
	def canStartInsurrection(self, objUnit):

		if(not g_bEnableGreatStatesmanStartInsurrection):
			return fales
				
		if(objUnit == None):
			return false
			
		if(objUnit.isNone()):
			return false

		if(objUnit.getUnitType() != gc.getInfoTypeForString("UNIT_STATESMAN")):
			return false
		
		objPlot = objUnit.plot()
		
		if(objPlot.getOwner() == objUnit.getOwner()):
			return false	

		if(objPlot.getOwner() != objUnit.getOwner() and not objPlot.isCity()):
			return false

		objCity = objPlot.getPlotCity()
		
		objPlayerTeam = gc.getTeam(objUnit.getTeam())
		
		if(not g_bStartInsurrectionInFriendlyCities and objPlot.getOwner() != objUnit.getOwner() and objPlot.isCity() and not objPlayerTeam.isAtWar(objCity.getOwner())):
			return false

		if(not g_bStartInsurrectionInPlayerCapitals and objCity.isCapital()):
			return false

		return true
												
												
	# This method will handle the case where the improve diplomatic relations action button is pressed.
	def eventDiplomaticsRelationsPopup(self): 
		player = gc.getActivePlayer()
		iPlayer = player.getID()
		
		popup = PyPopup.PyPopup(DIPLOMATIC_RELATIONS_POPUP, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString("Select a leader to improve diplomatic relations with:")

		# Go through all of the players in the game.
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):

			# Only allow players that are alive, not the current active player or a minor civilization
			if (gc.getPlayer(iLoopPlayer).isAlive() and iLoopPlayer != iPlayer and not gc.getPlayer(iLoopPlayer).isBarbarian() and  not gc.getPlayer(iLoopPlayer).isMinorCiv()):

				# Add any players that have met the currently active player to the popup
				if (gc.getTeam(gc.getPlayer(iLoopPlayer).getTeam()).isHasMet(player.getTeam())):

					gc.getPlayer(iLoopPlayer).AI_setAttitudeExtra(iPlayer, 0)
					
					# Get the current player's attitude towards the active player
					strButtonText = str(gc.getAttitudeInfo(gc.getPlayer(iLoopPlayer).AI_getAttitude(iPlayer)).getDescription())
					
					strHelpText = str("Improve diplomatic relations with " + gc.getPlayer(iLoopPlayer).getName())
					
					# Get the information if the current player is at war with the currently active player
					if(gc.getTeam(gc.getPlayer(iLoopPlayer).getTeam()).isAtWar(player.getTeam())):
						strButtonText = strButtonText + " and at war"
						strHelpText = str("Make peace with " + gc.getPlayer(iLoopPlayer).getName())
						
					strButtonText = strButtonText + " with you."
					
					leaderHeadImage = str(gc.getLeaderHeadInfo( gc.getPlayer(iLoopPlayer).getLeaderType() ).getButton())					
					popup.addPythonButton( str(gc.getPlayer(iLoopPlayer).getName()), strButtonText, strHelpText, leaderHeadImage, iLoopPlayer, -1, true  )
					
		popup.launch(true)
	 	
	
	def	startInsurrection(self, objUnit):
				
		if(objUnit == None):
			return
			
		if(objUnit.isNone()):
			return
			
		if(objUnit.getUnitType() != gc.getInfoTypeForString("UNIT_STATESMAN")):
			return 
			
		targetPlot = objUnit.plot()
		
		targetCity = targetPlot.getPlotCity()
		
		iActivePlayer = objUnit.getOwner()
		iTargetPlayer = targetPlot.getOwner()
		targetPlayer = gc.getPlayer(iTargetPlayer)
		
		iActivePlayerCityCulture = float(targetPlot.getCulture(iActivePlayer))
		iTargetPlayerCityCulture = float(targetPlot.getCulture(iTargetPlayer))
		iTotalCityCulture = float(targetPlot.countTotalCulture())
		iActivePlayerPercentCulture = int((iActivePlayerCityCulture/iTotalCityCulture) * 100)
		
		iInsurrectionTime = g_iBaseInsurrectionTime
		
		if(g_bInsurrectionTimePlayerCultureBased):
			iInsurrectionTime = int(iInsurrectionTime + ((iTotalCityCulture/iTargetPlayerCityCulture) * (0.1 * iActivePlayerCityCulture)))

		if(g_bInsurrectionCanFlipCity):
			iFlipChance = gc.getGame().getMapRand().get(int(iTotalCityCulture),"Great Statesman")
			if(iActivePlayerCityCulture >= iFlipChance):
				targetPlot.setOwner(iActivePlayer)
				return
		
		targetCity.changeOccupationTimer(iInsurrectionTime)

		strMessage = objUnit.getNameNoDesc() + " has started an insurrection in " + targetCity.getName()
		
		# Inform the player that their great statesman has started an insurrection.
		CyInterface().addMessage(iActivePlayer, False, 0, strMessage, "", 0, gc.getUnitInfo(objUnit.getUnitType()).getButton(), ColorTypes(0), targetCity.getX(), targetCity.getY(), True, True) 

		objUnit.kill(true, objUnit.getOwner())
